@@ -34,7 +34,8 @@ from lib389.encrypted_attributes import EncryptedAttr, EncryptedAttrs
 # This is for sample entry creation.
 from lib389.configurations import get_sample_entries
 
-from lib389.lint import DSBLE0001, DSBLE0002, DSBLE0003, DSBLE0004, DSBLE0005, DSBLE0006, DSVIRTLE0001, DSCLLE0001
+from lib389.lint import DSBLE0001, DSBLE0002, DSBLE0003, DSBLE0004, DSBLE0005, DSBLE0006, DSBLE0007, DSVIRTLE0001, DSCLLE0001
+from lib389.plugins import USNPlugin
 
 
 class BackendLegacy(object):
@@ -607,6 +608,160 @@ class Backend(DSLdapObject):
             self._log.debug(f"_lint_cl_trimming - backend ({suffix}) is not replicated")
             pass
 
+    def _lint_system_indexes(self):
+        """Check that system indexes are correctly configured"""
+        bename = self.lint_uid()
+        suffix = self.get_attr_val_utf8('nsslapd-suffix')
+        indexes = self.get_indexes()
+
+        # Default system indexes taken from ldap/servers/slapd/back-ldbm/instance.c
+        expected_system_indexes = {
+            'entryrdn': {'types': ['subtree'], 'matching_rule': None},
+            'parentid': {'types': ['eq'], 'matching_rule': 'integerOrderingMatch', 'scanlimit': 'limit=5000 type=eq flags=AND'},
+            'ancestorid': {'types': ['eq'], 'matching_rule': 'integerOrderingMatch', 'scanlimit': 'limit=5000 type=eq flags=AND'},
+            'objectClass': {'types': ['eq'], 'matching_rule': None},
+            'aci': {'types': ['pres'], 'matching_rule': None},
+            'nscpEntryDN': {'types': ['eq'], 'matching_rule': None},
+            'nsUniqueId': {'types': ['eq'], 'matching_rule': None},
+            'nsds5ReplConflict': {'types': ['eq', 'pres'], 'matching_rule': None}
+        }
+
+        # Default system indexes taken from ldap/ldif/template-dse.ldif.in
+        expected_system_indexes.update({
+            'nsCertSubjectDN': {'types': ['eq'], 'matching_rule': None},
+            'numsubordinates': {'types': ['pres'], 'matching_rule': None},
+            'nsTombstoneCSN': {'types': ['eq'], 'matching_rule': None},
+            'targetuniqueid': {'types': ['eq'], 'matching_rule': None}
+        })
+
+
+        # RetroCL plugin creates its own backend with an additonal index for changeNumber
+        # See ldap/servers/plugins/retrocl/retrocl_create.c
+        if suffix.lower() == 'cn=changelog':
+            expected_system_indexes.update({
+                'changeNumber': {'types': ['eq'], 'matching_rule': 'integerOrderingMatch'}
+            })
+
+        # USN plugin requires entryusn attribute indexed for equality with integerOrderingMatch rule
+        # See ldap/ldif/template-dse.ldif.in
+        try:
+            usn_plugin = USNPlugin(self._instance)
+            if usn_plugin.status():
+                expected_system_indexes.update({
+                    'entryusn': {'types': ['eq'], 'matching_rule': 'integerOrderingMatch'}
+                })
+        except Exception as e:
+            self._log.debug(f"_lint_system_indexes - Error checking USN plugin: {e}")
+
+        discrepancies = []
+        remediation_commands = []
+        reindex_attrs = set()
+
+        for attr_name, expected_config in expected_system_indexes.items():
+            try:
+                index = indexes.get(attr_name)
+            except ldap.NO_SUCH_OBJECT:
+                # Index is missing
+                index = None
+            except Exception as e:
+                self._log.debug(f"_lint_system_indexes - Error getting index {attr_name}: {e}")
+                discrepancies.append(f"Unable to check index {attr_name}: {str(e)}")
+                continue
+
+            try:
+                # Check if index exists
+                if index is None:
+                    discrepancies.append(f"Missing system index: {attr_name}")
+                    # Generate remediation command
+                    index_types = ' '.join([f"--index-type {t}" for t in expected_config['types']])
+                    cmd = f"dsconf YOUR_INSTANCE backend index add {bename} --attr {attr_name} {index_types}"
+                    if expected_config.get('matching_rule'):
+                        cmd += f" --matching-rule {expected_config['matching_rule']}"
+                    if expected_config.get('scanlimit'):
+                        cmd += f" --add-scanlimit {expected_config['scanlimit']}"
+                    remediation_commands.append(cmd)
+                    reindex_attrs.add(attr_name)  # New index needs reindexing
+                else:
+                    # Index exists, check configuration
+                    actual_types = index.get_attr_vals_utf8('nsIndexType') or []
+                    actual_mrs = index.get_attr_vals_utf8('nsMatchingRule') or []
+                    actual_scanlimit = index.get_attr_vals_utf8('nsIndexIDListScanLimit') or []
+
+                    # Normalize to lowercase for comparison
+                    actual_types = [t.lower() for t in actual_types]
+                    expected_types = [t.lower() for t in expected_config['types']]
+
+                    # Check index types
+                    missing_types = set(expected_types) - set(actual_types)
+                    if missing_types:
+                        discrepancies.append(f"Index {attr_name} missing types: {', '.join(missing_types)}")
+                        missing_type_args = ' '.join([f"--add-type {t}" for t in missing_types])
+                        cmd = f"dsconf YOUR_INSTANCE backend index set {bename} --attr {attr_name} {missing_type_args}"
+                        remediation_commands.append(cmd)
+                        reindex_attrs.add(attr_name)
+
+                    # Check matching rules
+                    expected_mr = expected_config.get('matching_rule')
+                    if expected_mr:
+                        actual_mrs_lower = [mr.lower() for mr in actual_mrs]
+                        if expected_mr.lower() not in actual_mrs_lower:
+                            discrepancies.append(f"Index {attr_name} missing matching rule: {expected_mr}")
+                            # Add the missing matching rule
+                            cmd = f"dsconf YOUR_INSTANCE backend index set {bename} --attr {attr_name} --add-mr {expected_mr}"
+                            remediation_commands.append(cmd)
+                            reindex_attrs.add(attr_name)
+
+                    # Check fine grain definitions for parentid ONLY
+                    expected_scanlimit = expected_config.get('scanlimit')
+                    if (attr_name.lower() == "parentid") and expected_scanlimit and (len(actual_scanlimit) == 0):
+                            discrepancies.append(f"Index {attr_name} missing fine grain definition of IDs limit: {expected_mr}")
+                            # Add the missing scanlimit
+                            if expected_mr:
+                                cmd = f"dsconf YOUR_INSTANCE backend index set {bename} --attr {attr_name} --add-mr {expected_mr} --add-scanlimit {expected_scanlimit}"
+                            else:
+                                cmd = f"dsconf YOUR_INSTANCE backend index set {bename} --attr {attr_name} --add-scanlimit {expected_scanlimit}"
+                            remediation_commands.append(cmd)
+                            reindex_attrs.add(attr_name)
+
+            except Exception as e:
+                self._log.debug(f"_lint_system_indexes - Error checking index {attr_name}: {e}")
+                discrepancies.append(f"Unable to check index {attr_name}: {str(e)}")
+
+        if discrepancies:
+            report = copy.deepcopy(DSBLE0007)
+            report['check'] = f'backends:{bename}:system_indexes'
+            report['items'] = [suffix]
+
+            expected_indexes_list = []
+            for attr_name, config in expected_system_indexes.items():
+                types_str = "', '".join(config['types'])
+                index_desc = f"- {attr_name}: index type{'s' if len(config['types']) > 1 else ''} '{types_str}'"
+                if config['matching_rule']:
+                    index_desc += f" with matching rule '{config['matching_rule']}'"
+                expected_indexes_list.append(index_desc)
+
+            formatted_expected_indexes = '\n'.join(expected_indexes_list)
+            report['detail'] = report['detail'].replace('EXPECTED_INDEXES', formatted_expected_indexes)
+            report['detail'] = report['detail'].replace('DISCREPANCIES', '\n'.join([f"- {d}" for d in discrepancies]))
+
+            formatted_commands = '\n'.join([f"    # {cmd}" for cmd in remediation_commands])
+            report['fix'] = report['fix'].replace('REMEDIATION_COMMANDS', formatted_commands)
+
+            # Generate specific reindex commands for affected attributes
+            if reindex_attrs:
+                reindex_commands = []
+                for attr in sorted(reindex_attrs):
+                    reindex_cmd = f"dsconf YOUR_INSTANCE backend index reindex {bename} --attr {attr}"
+                    reindex_commands.append(f"    # {reindex_cmd}")
+                formatted_reindex_commands = '\n'.join(reindex_commands)
+            else:
+                formatted_reindex_commands = "    # No reindexing needed"
+
+            report['fix'] = report['fix'].replace('REINDEX_COMMANDS', formatted_reindex_commands)
+            report['fix'] = report['fix'].replace('YOUR_INSTANCE', self._instance.serverid)
+            report['fix'] = report['fix'].replace('BACKEND_NAME', bename)
+            yield report
+
     def create_sample_entries(self, version):
         """Creates sample entries under nsslapd-suffix value
 
@@ -694,24 +849,32 @@ class Backend(DSLdapObject):
             parent_suffix = properties.pop('parent', False)
 
         # Okay, now try to make the backend.
-        super(Backend, self).create(dn, properties, basedn)
+        backend_obj = super(Backend, self).create(dn, properties, basedn)
 
         # We check if the mapping tree exists in create, so do this *after*
         if create_mapping_tree is True:
-            properties = {
+            mapping_tree_properties = {
                 'cn': self._nprops_stash['nsslapd-suffix'],
                 'nsslapd-state': 'backend',
                 'nsslapd-backend': self._nprops_stash['cn'],
             }
             if parent_suffix:
                 # This is a subsuffix, set the parent suffix
-                properties['nsslapd-parent-suffix'] = parent_suffix
-            self._mts.create(properties=properties)
+                mapping_tree_properties['nsslapd-parent-suffix'] = parent_suffix
+
+            try:
+                self._mts.create(properties=mapping_tree_properties)
+            except Exception as e:
+                try:
+                    backend_obj.delete()
+                except Exception as cleanup_error:
+                    self._instance.log.error(f"Failed to cleanup backend after mapping tree creation failure: {cleanup_error}")
+                raise e
 
         # We can't create the sample entries unless a mapping tree was installed.
         if sample_entries is not False and create_mapping_tree is True:
             self.create_sample_entries(sample_entries)
-        return self
+        return backend_obj
 
     def delete(self):
         """Deletes the backend, it's mapping tree and all related indices.
@@ -797,12 +960,13 @@ class Backend(DSLdapObject):
                 return
         raise ValueError("Can not delete index because it does not exist")
 
-    def add_index(self, attr_name, types, matching_rules=None, reindex=False):
+    def add_index(self, attr_name, types, matching_rules=None, idlistscanlimit=None, reindex=False):
         """ Add an index.
 
         :param attr_name - name of the attribute to index
         :param types - a List of index types(eq, pres, sub, approx)
         :param matching_rules - a List of matching rules for the index
+        :param idlistscanlimit - a List of fine grain definitions for scanning limit
         :param reindex - If set to True then index the attribute after creating it.
         """
 
@@ -832,6 +996,15 @@ class Backend(DSLdapObject):
             # Only add if there are actually rules present in the list.
             if len(mrs) > 0:
                 props['nsMatchingRule'] = mrs
+
+        if idlistscanlimit is not None:
+            scanlimits = []
+            for scanlimit in idlistscanlimit:
+                scanlimits.append(scanlimit)
+            # Only add if there are actually limits in the list.
+            if len(scanlimits) > 0:
+                props['nsIndexIDListScanLimit'] = scanlimits
+
         new_index.create(properties=props, basedn="cn=index," + self._dn)
 
         if reindex:
@@ -1138,6 +1311,7 @@ class DatabaseConfig(DSLdapObject):
             'nsslapd-lookthroughlimit',
             'nsslapd-mode',
             'nsslapd-idlistscanlimit',
+            'nsslapd-systemidlistscanlimit',
             'nsslapd-directory',
             'nsslapd-import-cachesize',
             'nsslapd-idl-switch',
@@ -1153,6 +1327,10 @@ class DatabaseConfig(DSLdapObject):
             'nsslapd-db-durable-transaction',
             'nsslapd-search-bypass-filter-test',
             'nsslapd-serial-lock',
+            'nsslapd-dynamic-lists-enabled',
+            'nsslapd-dynamic-lists-attr',
+            'nsslapd-dynamic-lists-oc',
+            'nsslapd-dynamic-lists-url-attr',
         ]
         self._db_attrs = {
             'bdb':
@@ -1184,6 +1362,7 @@ class DatabaseConfig(DSLdapObject):
                     'nsslapd-mdb-max-size',
                     'nsslapd-mdb-max-readers',
                     'nsslapd-mdb-max-dbs',
+                    'nsslapd-cache-autosize',
                 ]
         }
         self._create_objectclasses = ['top', 'extensibleObject']
@@ -1248,6 +1427,7 @@ class BackendSuffixView(CompositeDSLdapObject):
         be_args = [
             'nsslapd-cachememsize',
             'nsslapd-cachesize',
+            'nsslapd-cache-preserved-entries',
             'nsslapd-dncachememsize',
             'nsslapd-readonly',
             'nsslapd-require-index',
