@@ -18,7 +18,7 @@ from lib389.configurations.sample import (
 from lib389.chaining import (ChainingLinks)
 from lib389.monitor import MonitorLDBM
 from lib389.replica import Replicas
-from lib389.utils import ensure_str, is_a_dn, is_dn_parent
+from lib389.utils import ensure_str, is_a_dn, is_dn_parent, parse_size, align_to_page_size
 from lib389.tasks import DBCompactTask
 from lib389._constants import INSTALL_LATEST_CONFIG
 from lib389.properties import BACKEND_SAMPLE_ENTRIES
@@ -39,6 +39,7 @@ arg_to_attr = {
         'mode': 'nsslapd-mode',
         'state': 'nsslapd-state',
         'idlistscanlimit': 'nsslapd-idlistscanlimit',
+        'systemidlistscanlimit': 'nsslapd-systemidlistscanlimit',
         'directory': 'nsslapd-directory',
         'dbcachesize': 'nsslapd-dbcachesize',
         'logdirectory': 'nsslapd-db-logdirectory',
@@ -74,6 +75,12 @@ arg_to_attr = {
         'search_scope': 'vlvscope',
         'search_filter': 'vlvfilter',
         'sort': 'vlvsort',
+        # Dynamic lists
+        'enable_dynamic_lists': 'nsslapd-dynamic-lists-enabled',
+        'disable_dynamic_lists': 'nsslapd-dynamic-lists-disabled',
+        'dynamic_oc': 'nsslapd-dynamic-lists-oc',
+        'dynamic_list_attr': 'nsslapd-dynamic-lists-attr',
+        'dynamic_url_attr': 'nsslapd-dynamic-lists-url-attr',
     }
 
 SINGULAR = Backend
@@ -504,6 +511,8 @@ def backend_set(inst, basedn, log, args):
         bev.set('nsslapd-cachesize', args.cache_size)
     if args.cache_memsize:
         bev.set('nsslapd-cachememsize', args.cache_memsize)
+    if args.cache_preserved_entries:
+        bev.set('nsslapd-cache-preserved-entries', args.cache_preserved_entries)
     if args.dncache_memsize:
         bev.set('nsslapd-dncachememsize', args.dncache_memsize)
     if args.require_index:
@@ -535,12 +544,39 @@ def db_config_set(inst, basedn, log, args):
     did_something = False
     replace_list = []
 
+    if getattr(args,'enable_dynamic_lists', None) and getattr(args, 'disable_dynamic_lists', None):
+        raise ValueError("You can not enable and disable dynamic lists at the same time")
+
     for attr, value in list(attrs.items()):
         if value == "":
             # We don't support deleting attributes or setting empty values in db
             continue
-        else:
-            replace_list.append([attr, value])
+
+        if attr == "nsslapd-dynamic-lists-enabled":
+            if value:
+                value = "on"
+            else:
+                continue
+        elif attr == "nsslapd-dynamic-lists-disabled":
+            attr = "nsslapd-dynamic-lists-enabled"
+            if value:
+                value = "off"
+            else:
+                continue
+
+        if attr == "nsslapd-mdb-max-size":
+            try:
+                mdb_max_size = parse_size(value)
+                # MDB max size requires pagesize alignment
+                mdb_max_size_aligned = align_to_page_size(mdb_max_size)
+                if mdb_max_size_aligned != mdb_max_size:
+                    log.info(f"Aligning MDB max size from {mdb_max_size} to nearest pagesize {mdb_max_size_aligned}")
+                value = str(mdb_max_size_aligned)
+            except ValueError:
+                raise ValueError(f"Invalid value for --mdb-max-size: {value}")
+
+        replace_list.append([attr, value])
+
     if len(replace_list) > 0:
         db_cfg.set(replace_list)
     elif not did_something:
@@ -592,6 +628,21 @@ def backend_set_index(inst, basedn, log, args):
             except ldap.NO_SUCH_ATTRIBUTE:
                 raise ValueError('Can not delete matching rule type because it does not exist')
 
+    if args.replace_scanlimit is not None:
+        for replace_scanlimit in args.replace_scanlimit:
+            index.replace('nsIndexIDListScanLimit', replace_scanlimit)
+
+    if args.add_scanlimit is not None:
+        for add_scanlimit in args.add_scanlimit:
+            index.add('nsIndexIDListScanLimit', add_scanlimit)
+
+    if args.del_scanlimit is not None:
+        for del_scanlimit in args.del_scanlimit:
+            try:
+                index.remove('nsIndexIDListScanLimit', del_scanlimit)
+            except ldap.NO_SUCH_ATTRIBUTE:
+                raise ValueError('Can not delete a fine grain limit definition because it does not exist')
+
     if args.reindex:
         be.reindex(attrs=[args.attr])
     log.info("Index successfully updated")
@@ -886,6 +937,7 @@ def create_parser(subparsers):
     set_backend_parser.add_argument('--disable', action='store_true', help='Disables the backend database')
     set_backend_parser.add_argument('--cache-size', help='Sets the maximum number of entries to keep in the entry cache')
     set_backend_parser.add_argument('--cache-memsize', help='Sets the maximum size in bytes that the entry cache can grow to')
+    set_backend_parser.add_argument('--cache-preserved-entries', help='Sets the maximum number of entries that are not evicted from the cache when trying to make space. This is typically used to keep very large groups in the cache')
     set_backend_parser.add_argument('--dncache-memsize', help='Sets the maximum size in bytes that the DN cache can grow to')
     set_backend_parser.add_argument('--state', help='Changes the backend state to: "backend", "disabled", "referral", or "referral on update"')
     set_backend_parser.add_argument('be_name', help='The backend name or suffix')
@@ -913,6 +965,9 @@ def create_parser(subparsers):
     edit_index_parser.add_argument('--del-type', action='append', help='Removes an index type from the index: (eq, sub, pres, or approx)')
     edit_index_parser.add_argument('--add-mr', action='append', help='Adds a matching-rule to the index')
     edit_index_parser.add_argument('--del-mr', action='append', help='Removes a matching-rule from the index')
+    edit_index_parser.add_argument('--add-scanlimit', action='append', help='Adds a fine grain limit definiton to the index')
+    edit_index_parser.add_argument('--replace-scanlimit', action='append', help='Replaces a fine grain limit definiton to the index')
+    edit_index_parser.add_argument('--del-scanlimit', action='append', help='Removes a fine grain limit definiton to the index')
     edit_index_parser.add_argument('--reindex', action='store_true', help='Re-indexes the database after editing the index')
     edit_index_parser.add_argument('be_name', help='The backend name or suffix')
 
@@ -1039,6 +1094,7 @@ def create_parser(subparsers):
                                                                  'will check when examining candidate entries in response to a search request')
     set_db_config_parser.add_argument('--mode', help='Specifies the permissions used for newly created index files')
     set_db_config_parser.add_argument('--idlistscanlimit', help='Specifies the number of entry IDs that are searched during a search operation')
+    set_db_config_parser.add_argument('--systemidlistscanlimit', help='Specifies the number of entry IDs that are fetch from ancestorid/parentid indexes')
     set_db_config_parser.add_argument('--directory', help='Specifies absolute path to database instance')
     set_db_config_parser.add_argument('--dbcachesize', help='Specifies the database index cache size in bytes')
     set_db_config_parser.add_argument('--logdirectory', help='Specifies the path to the directory that contains the database transaction logs')
@@ -1065,9 +1121,9 @@ def create_parser(subparsers):
     set_db_config_parser.add_argument('--locks-monitoring-pause', help='Sets the DB lock monitoring value in milliseconds for the amount of time '
                                                                        'that the monitoring thread spends waiting between checks.')
     set_db_config_parser.add_argument('--import-cache-autosize', help='Enables or disables to automatically set the size of the import '
-                                                                       'cache to be used during the import process of LDIF files')
+                                                                      'cache to be used during the import process of LDIF files')
     set_db_config_parser.add_argument('--cache-autosize', help='Sets the percentage of free memory that is used in total for the database '
-                                                               'and entry cache. "0" disables this feature.')
+                                                               'and entry cache.')
     set_db_config_parser.add_argument('--cache-autosize-split', help='Sets the percentage of RAM that is used for the database cache. The '
                                                                      'remaining percentage is used for the entry cache')
     set_db_config_parser.add_argument('--import-cachesize', help='Sets the size in bytes of the database cache used in the import process.')
@@ -1085,10 +1141,15 @@ def create_parser(subparsers):
     set_db_config_parser.add_argument('--deadlock-policy', help='Adjusts the backend database deadlock policy (Advanced setting)')
     set_db_config_parser.add_argument('--db-home-directory', help='Sets the directory for the database mmapped files (Advanced setting)')
     set_db_config_parser.add_argument('--db-lib', help='Sets which db lib is used. Valid values are: bdb or mdb')
-    set_db_config_parser.add_argument('--mdb-max-size', help='Sets the lmdb database maximum size (in bytes).')
+    set_db_config_parser.add_argument('--mdb-max-size', help='Sets the lmdb database maximum size (accepts bytes, or with unit suffix: k, m, g, t)')
     set_db_config_parser.add_argument('--mdb-max-readers', help='Sets the lmdb database maximum number of readers (Advanced setting)')
     set_db_config_parser.add_argument('--mdb-max-dbs', help='Sets the lmdb database maximum number of sub databases (Advanced setting)')
-
+    # Dynamic lists
+    set_db_config_parser.add_argument('--enable-dynamic-lists', action='store_true', help='Enables dynamic lists')
+    set_db_config_parser.add_argument('--disable-dynamic-lists', action='store_true', help='Disables dynamic lists')
+    set_db_config_parser.add_argument('--dynamic-oc', help='Sets the objectclass for dynamic lists')
+    set_db_config_parser.add_argument('--dynamic-url-attr', help='Sets the url attribute for dynamic lists')
+    set_db_config_parser.add_argument('--dynamic-list-attr', help='Sets the list attribute for dynamic lists')
 
     #######################################################
     # Database & Suffix Monitor
