@@ -11,26 +11,16 @@
 
     TODO put them in a module!
 """
-try:
-    from subprocess import Popen as my_popen, PIPE
-except ImportError:
-    from popen2 import popen2
-
-    def my_popen(cmd_l, stdout=None):
-        class MockPopenResult(object):
-            def wait(self):
-                pass
-        p = MockPopenResult()
-        p.stdout, p.stdin = popen2(cmd_l)
-        return p
-
 import json
 import re
 import os
+import glob
 import logging
 import shutil
 import ldap
+import mmap
 import socket
+import ipaddress
 import time
 import stat
 from datetime import (datetime, timedelta)
@@ -54,7 +44,8 @@ from lib389.paths import ( Paths, DEFAULTS_PATH )
 from lib389.dseldif import DSEldif
 from lib389._constants import (
         DEFAULT_USER, VALGRIND_WRAPPER, DN_CONFIG, CFGSUFFIX, LOCALHOST,
-        ReplicaRole, CONSUMER_REPLICAID, SENSITIVE_ATTRS, DEFAULT_DB_LIB
+        ReplicaRole, CONSUMER_REPLICAID, SENSITIVE_ATTRS, DEFAULT_DB_LIB,
+        DEFAULT_LMDB_SIZE
     )
 from lib389.properties import (
         SER_HOST, SER_USER_ID, SER_GROUP_ID, SER_STRICT_HOSTNAME_CHECKING, SER_PORT,
@@ -180,6 +171,16 @@ _chars = {
     "~": u"\u02de",
 }
 
+#
+# Size parser constants
+#
+SIZE_UNITS = { 't': 2**40, 'g': 2**30, 'm': 2**20, 'k': 2**10, '': 1, }
+SIZE_PATTERN = r'\s*(\d*\.?\d*)\s*([tgmk]?)b?\s*'
+
+RPM_TOOL = '/usr/bin/rpm'
+LDD_TOOL = '/usr/bin/ldd'
+OBJDUMP_TOOL = '/usr/bin/objdump'
+
 #
 # Utilities
 #
@@ -187,7 +188,7 @@ _chars = {
 
 def selinux_present():
     """
-    Determine if selinux libraries are on a system, and if so, if we are in
+    Determine if SELinux libraries are on a system, and if so, if we are in
     a state to consume them (enabled, disabled).
 
     :returns: bool
@@ -197,20 +198,20 @@ def selinux_present():
     try:
         import selinux
         if selinux.is_selinux_enabled():
-            # We have selinux, lets see if we are allowed to configure it.
+            # We have SELinux, lets see if we are allowed to configure it.
             # (just checking the uid for now - we may rather check if semanage command success)
             if os.geteuid() != 0:
-                log.info('Non privileged user cannot use semanage, will not relabel ports or files.' )
+                log.info('Non-privileged user cannot use semanage, will not relabel ports or files.' )
             elif not os.path.exists('/usr/sbin/semanage'):
                 log.info('semanage command is not installed, will not relabel ports or files. Please install policycoreutils-python-utils.' )
             else:
                 status = True
         else:
             # We have the module, but it's disabled.
-            log.info('selinux is disabled, will not relabel ports or files.' )
+            log.info('SELinux is disabled, will not relabel ports or files.' )
     except ImportError:
         # No python module, so who knows what state we are in.
-        log.error('selinux python module not found, will not relabel files.' )
+        log.error('SELinux python module not found, will not relabel files.' )
 
     return status
 
@@ -226,11 +227,11 @@ def selinux_restorecon(path):
     try:
         import selinux
     except ImportError:
-        log.debug('selinux python module not found, skipping relabel path %s' % path)
+        log.debug('SELinux python module not found, skipping relabel path %s' % path)
         return
 
     if not selinux.is_selinux_enabled():
-        log.debug('selinux is disabled, skipping relabel path %s' % path)
+        log.debug('SELinux is disabled, skipping relabel path %s' % path)
         return
 
     try:
@@ -313,7 +314,7 @@ def selinux_label_file(path, label):
         rc = 0
         for i in range(5):
             try:
-                log.debug(f"Setting label {label} in SELinux file context {path}.  Attempt {i}")
+                log.debug(f"Setting label {label} in SELinux file context {path}. Attempt {i}")
                 result = subprocess.run(["semanage", "fcontext", "-a", "-t", label, path],
                                          stdout=subprocess.PIPE,
                                          stderr=subprocess.PIPE)
@@ -368,7 +369,7 @@ def selinux_clean_ports_label():
 
 
 def _get_selinux_port_policies(port):
-    """Get a list of selinux port policies for the specified port, 'tcp' protocol and
+    """Get a list of SELinux port policies for the specified port, 'tcp' protocol and
     excluding 'unreserved_port_t', 'reserved_port_t', 'ephemeral_port_t' labels"""
 
     # [2:] - cut first lines containing the headers. [:-1] - empty line
@@ -406,11 +407,11 @@ def selinux_label_port(port, remove_label=False):
     try:
         import selinux
     except ImportError:
-        log.debug('selinux python module not found, skipping port labeling.')
+        log.debug('SELinux python module not found, skipping port labeling.')
         return
 
     if not selinux_present():
-        log.debug('selinux is disabled, skipping port relabel')
+        log.debug('SELinux is disabled, skipping port relabel')
         return
 
     # We only label ports that ARE NOT in the default policy that comes with
@@ -611,7 +612,7 @@ def valgrind_enable(sbin_dir, wrapper=None):
     (making a backup of the original ns-slapd binary).
 
     The script calling valgrind_enable() must be run as the 'root' user
-    as selinux needs to be disabled for valgrind to work
+    as SELinux needs to be disabled for valgrind to work
 
     The server instance(s) should be stopped prior to calling this function.
     Then after calling valgrind_enable():
@@ -678,7 +679,7 @@ def valgrind_enable(sbin_dir, wrapper=None):
         raise IOError('failed to copy valgrind wrapper to ns-slapd, error: %s' %
                       e.strerror)
 
-    # Disable selinux
+    # Disable SELinux
     if os.geteuid() == 0:
         os.system('setenforce 0')
 
@@ -702,7 +703,7 @@ def valgrind_disable(sbin_dir):
     Restore the ns-slapd binary to its original state - the server instances
     are expected to be stopped.
 
-    Note - selinux is enabled at the end of this process.
+    Note - SELinux is enabled at the end of this process.
 
     :param sbin_dir - the location of the ns-slapd binary (e.g. /usr/sbin)
     :raise ValueError
@@ -735,7 +736,7 @@ def valgrind_disable(sbin_dir):
         raise ValueError('Failed to delete backup ns-slapd, error: %s' %
                          e.strerror)
 
-    # Enable selinux
+    # Enable SELinux
     if os.geteuid() == 0:
         os.system('setenforce 1')
 
@@ -845,7 +846,7 @@ def isLocalHost(host_name):
 
     # next, see if this IP addr is one of our
     # local addresses
-    p = my_popen(['/sbin/ip', 'addr'], stdout=PIPE)
+    p = subprocess.Popen(['/sbin/ip', 'addr'], stdout=subprocess.PIPE)
     child_stdout = p.stdout.read()
     found = ('inet %s' % ip_addr).encode() in child_stdout
     p.wait()
@@ -1487,6 +1488,11 @@ def ensure_dict_str(val):
     return retdict
 
 
+def align_to_page_size(size):
+    page_size = os.sysconf("SC_PAGE_SIZE")
+    return ((size + page_size - 1) // page_size) * page_size
+
+
 def pseudolocalize(string):
     pseudo_string = u""
     for char in string:
@@ -1700,11 +1706,115 @@ def is_valid_hostname(hostname):
     allowed = re.compile(r"(?!-)[A-Z\d-]{1,63}(?<!-)$", re.IGNORECASE)
     return all(allowed.match(x) for x in hostname.split("."))
 
+def is_valid_ip(ip):
+    """ Validate an IPv4 or IPv6 address, including asterisks for wildcards. """
+    if '*' in ip and '.' in ip:
+        ipv4_pattern = r'^(\d{1,3}|\*)\.(\d{1,3}|\*)\.(\d{1,3}|\*)\.(\d{1,3}|\*)$'
+        if re.match(ipv4_pattern, ip):
+            octets = ip.split('.')
+            for octet in octets:
+                if octet != '*':
+                    try:
+                        val = int(octet, 10)
+                        if not (0 <= val <= 255):
+                            return False
+                    except ValueError:
+                        return False
+            return True
+        else:
+            return False
+
+    if '*' in ip and ':' in ip:
+        ipv6_pattern = r'^([0-9a-fA-F]{1,4}|\*)(:([0-9a-fA-F]{1,4}|\*)){0,7}$'
+        if re.match(ipv6_pattern, ip):
+            octets = ip.split(':')
+            for octet in octets:
+                if octet != '*':
+                    try:
+                        val = int(octet, 16)
+                        if not (0 <= val <= 0xFFFF):
+                            return False
+                    except ValueError:
+                        return False
+            return True
+        else:
+            return False
+
+    try:
+        ipaddress.ip_address(ip)
+        return True
+    except ValueError:
+        return False
+
+def parse_size(size):
+    """
+    Parse a string representing a size (like "5 kb" or "2.5Gb") and
+    return the size in bytes.
+    :param size: The size to parse
+    :type size: str:
+    :return int:
+    :raise ValueError: if the string cannot be parsed.
+    """
+    try:
+        val,unit = re.fullmatch(SIZE_PATTERN, str(size), flags=re.IGNORECASE).groups()
+        return round(float(val) * SIZE_UNITS[unit.lower()])
+    except AttributeError:
+        raise ValueError(f'Unable to parse "{size}" as a size.')
+
+
+def format_size(size):
+    """
+    Return a string representing a size (like "5 kb" or "2.5Gb")
+    :param size: The size int bytes to format
+    :type size: int:
+    :return str:
+    """
+    for unit in ["B", "KB", "MB", "GB",]:
+        if (size < 1024):
+            return f"{size:3.1f} {unit}"
+        size /= 1024
+    return f"{size:.1f} TB"
+
 
 def get_default_db_lib():
+    """
+    Get the default value for the database implementation
+    :return str:  Should be either 'bdb' or 'mdb'
+    """
     return os.getenv('NSSLAPD_DB_LIB', default=DEFAULT_DB_LIB)
 
 
+def get_default_mdb_max_size(paths):
+    """
+    Get the default maximum size for the lmdb database.
+    :return str:  A size that can be parsed with parse_size()
+    """
+    if paths is None:
+        paths = Paths()
+    mdb_max_size = format_size(parse_size(DEFAULT_LMDB_SIZE))
+    size = parse_size(mdb_max_size)
+    # Make sure that there is enough available disk space
+    # otherwise decrease the value
+    dbdir = paths.db_dir
+    # dbdir may not exists because:
+    #  - paths instance name has not been expanded
+    #  - dscreate has not yet created it.
+    # so let use the first existing parent in that case.
+    while not os.path.exists(dbdir):
+        dbdir = os.path.dirname(dbdir)
+    try:
+        statvfs = os.statvfs(dbdir)
+        avail = statvfs.f_frsize * statvfs.f_bavail
+        avail *= 0.8 # Reserve 20% as margin
+        if size > avail:
+            mdb_max_size = format_size(avail)
+    except (TimeoutError, InterruptedError) as e:
+        raise e
+    except OSError as e:
+        log.warning(f'Cannot determine the free space in the file system containing {dbdir} because of {e}')
+    return mdb_max_size
+
+
 def is_fips():
     if os.path.exists('/proc/sys/crypto/fips_enabled'):
         with open('/proc/sys/crypto/fips_enabled', 'r') as f:
@@ -1912,3 +2022,44 @@ def get_passwd_from_file(passwd_file):
             passwd = f.readline().strip()
             return passwd
     raise ValueError(f"The password file '{passwd_file}' does not exist, or can not be read.")
+
+
+def find_plugin_path(plugin_name):
+    p = Paths()
+    plugin = None
+    for ppath in glob.glob(f'{p.plugin_dir}/{plugin_name}.*'):
+        if not ppath.endswith('.la'):
+            plugin = ppath
+    return plugin
+
+
+def check_plugin_strings(plugin_name, tested_strings):
+    """
+    Returns a dict mapping each tested symbol to True, False or None
+    """
+    plugin = find_plugin_path(plugin_name)
+    if plugin:
+        # Otherwise looks directly for the string in the plugin
+        with open(plugin, "rb") as f:
+            with mmap.mmap(f.fileno(), 0, access=mmap.ACCESS_READ) as mm:
+                return { astring:(mm.find(astring.encode()) >=0) for astring in tested_strings }
+    return { astring:None for astring in tested_strings }
+
+
+def get_timeout_scale():
+    """
+    Get the timeout scale factor
+    :return float:
+    """
+    scale_factor = 1.0
+    if Paths().asan_enabled:
+        scale_factor = 2.0
+
+    env_value = os.getenv('DS_TIMEOUT_SCALE', default=str(scale_factor))
+
+    try:
+        return float(env_value)
+    except ValueError:
+        log.error(f"DS_TIMEOUT_SCALE should be a valid float. Using default value: {scale_factor}")
+        return scale_factor
+
